/*
This sketch is designed to control my old heater ( Brand "Cannon"...junk avoid!). This heater has 4 switches to control it.
On/OFF
GAS LOW/HIGH
FAN LOW/MID
FAN MID/HIGH

In this fist cut, im just going to use "SetTemp" values from the vera device to control the heater in 4 
"steps" - im not going to use it with any feedback control or "thermostat" functionality as its a large room 
and getting the room temp isnt easy...

The proposed "steps"
1 is on/low/low
2 is on/mid/low
3 is on/mid/high
4 is on/high/high

Version Info
1.0 Inital working version

*/

#include <Sensor.h>
#include <SPI.h>
#include <EEPROM.h> 
#include <RF24.h>
#include <DallasTemperature.h>
#include <OneWire.h>


//Relay Pins
#define PWR_RLY 4
#define GAS_RLY 5
#define MID_RLY 6
#define HIGH_RLY 7

#define RELAY_ON 1
#define RELAY_OFF 0


//Sensor gw;
Sensor gw(8,9); //Rboard PCB uses different pins for CE/CS - also interupt, but this isnt used at this time.

#define CHILD_ID 1




// Setup a oneWire instance to communicate with any OneWire devices (not just Maxim/Dallas temperature ICs)
#define ONE_WIRE_BUS 2 // Pin where dallas sensor is connected 
OneWire oneWire(ONE_WIRE_BUS);
DallasTemperature sensors(&oneWire);
DeviceAddress heaterThermometer;




float lastTemperature ;
const int numReadings = 10;
int readings[numReadings];      // the readings from the analog input
int index = 0;                  // the index of the current reading
float total = 0;                  // the running total
float averageTemp = 0;                // the average
float averageTempA =0;

bool heaterStatus = 0;
int gasStatus = 0;
int fanStatus = 0;



 
void setup() 
{ 

  //Intialise the Dallas sensors
  sensors.begin();
  sensors.getAddress(heaterThermometer, 0);
  sensors.setResolution(heaterThermometer, 12);

 // initialize all the readings to 0: 
  for (int thisReading = 0; thisReading < numReadings; thisReading++)
    readings[thisReading] = 0;   


  //Intialise the MySensors code
  gw.begin();
  
  // Send the Sketch Version Information to the Gateway
  gw.sendSketchInfo("CannonHeater", "2507");
  gw.sendSensorPresentation(CHILD_ID, S_HEATER); 
  

//setup pins
     digitalWrite(PWR_RLY, RELAY_OFF);
     pinMode(PWR_RLY, OUTPUT);
     digitalWrite(GAS_RLY, RELAY_OFF);
     pinMode(GAS_RLY, OUTPUT);
     digitalWrite(MID_RLY, RELAY_OFF);
     pinMode(MID_RLY, OUTPUT);
     digitalWrite(HIGH_RLY, RELAY_OFF);
     pinMode(HIGH_RLY, OUTPUT);


}
 
void loop()
{

   
  if (gw.messageAvailable()) {
    message_s message = gw.getMessage(); 
    controlHeater(message);
  }
   
   getTemp ();
   
                     
         
}

 // Process incoming control messages
void controlHeater(message_s message) {

// Heater On Off control
  if (message.header.messageType=M_SET_VARIABLE &&
        message.header.type==V_HEATER_SW) {
         //Look for HeatOn message
          if  (strncmp(message.data,"HeatOn",6) == 0) { //strncmp is used to compare char array with string of length 6 
			 setHeaterOn ();
		   
        }
          //Look for Off message
          if  (strncmp(message.data,"Off",3) == 0) {
             setHeaterOff ();
           
        }
       }
        
// Fan Speed and Gas level control
  if (message.header.messageType=M_SET_VARIABLE &&
        message.header.type==V_HEATER) {
        int heatLvl = atoi(message.data);

    if ( heatLvl == 1) {
    Serial.print("Set heater LOW/LOW :");
      setFanLow();
      setGasLow() ;
      gw.sendVariable(CHILD_ID, V_HEATER, "1");  //report back to Vera
      }
    if ( heatLvl == 2 ) {
    Serial.print("Set heater MID/LOW :");
	setFanMid();
	setGasLow() ;
	gw.sendVariable(CHILD_ID, V_HEATER, "2");  //report back to Vera 
    }
    if ( heatLvl == 3 ) {
    Serial.print("Set heater MID/HIGH :");
      setFanMid();
      setGasHigh() ;
      gw.sendVariable(CHILD_ID, V_HEATER, "3");  //report back to Vera 
    }

    if ( heatLvl == 4 ) {
    Serial.print("Set heater HIGH/HIGH :");
      setFanHigh () ;
      setGasHigh () ;
      gw.sendVariable(CHILD_ID, V_HEATER, "4");  //report back to Vera 
   }   
   }
  }
  
void setHeaterOff () {
   Serial.println("Setting Heater OFF");
   digitalWrite(MID_RLY,RELAY_OFF); //also need to turn off the Mid speed relay
   digitalWrite(PWR_RLY,RELAY_OFF);
   gw.sendVariable(CHILD_ID, V_HEATER_SW, "Off");  //report back to Vera 
   heaterStatus = 0;
}
  
void setHeaterOn () {
   Serial.println("Setting Heater ON");
   digitalWrite(PWR_RLY,RELAY_ON);
   gw.sendVariable(CHILD_ID, V_HEATER_SW, "HeatOn");  //report back to Vera
   heaterStatus = 1;
}

void setGasLow () {
   digitalWrite(GAS_RLY,RELAY_OFF);
   Serial.println("Setting Gas LOW");
   gasStatus = 0;
}
  
void setGasHigh () {
   Serial.println("Setting Gas HIGH");
   digitalWrite(GAS_RLY,RELAY_ON);
   gasStatus = 1;
}

void setFanLow () {
   digitalWrite(MID_RLY,RELAY_OFF);
   digitalWrite(HIGH_RLY,RELAY_OFF);
   Serial.println("Setting Fan LOW");
   fanStatus = 0;
}

void setFanMid () {
   digitalWrite(MID_RLY,RELAY_ON);
   digitalWrite(HIGH_RLY,RELAY_OFF);
   Serial.println("Setting Fan MID");
   fanStatus = 1;
}

void setFanHigh () {
   digitalWrite(MID_RLY,RELAY_ON);
   digitalWrite(HIGH_RLY,RELAY_ON);
   Serial.println("Setting Fan HIGH");
   fanStatus = 2;
}


void getTemp () {
  //get temp and send to Heater device
   sensors.requestTemperatures(); // Fetch temperatures from Dallas
   delay(100);


//Perform smoothing on the temperature sensor data
  // subtract the last reading:
    total= total - readings[index];         
  // read from the sensor:  
    readings[index] = sensors.getTempC(heaterThermometer) ; 
  // add the reading to the total:
    total= total + readings[index];       
  // advance to the next position in the array:  
    index = index + 1;                    
  // if we're at the end of the array...
    if (index >= numReadings)              
    // ...wrap around to the beginning: 
    index = 0;                           

  // calculate the average:
    averageTemp = total / numReadings;         
  // send it to the computer as ASCII digits
    //Serial.println(averageTemp);   
   
  delay(1);        // delay in between reads for stability   
  
  
  if (averageTempA != averageTemp) {
      gw.sendVariable(CHILD_ID, V_TEMP, averageTemp,1);
      averageTempA=averageTemp;
      Serial.println(averageTemp);   
    }
}

	
	  
